import { promises } from "fs";
import { isAbsolute, resolve, dirname, join, parse } from "path";
import { transform, formatMessages } from "esbuild";
import { createFilter } from "@rollup/pluginutils";
const SCRIPT_LOADERS = ["js", "jsx", "ts", "tsx"];
const DEFAULT_EXCLUDE_REGEXP = /node_modules/;
const splitOptionsByType = (options) => {
  const inputOptions = [];
  const outputOptions = [];
  (Array.isArray(options) ? options : [options]).forEach(
    ({ output = false, ...commonOptions }) => {
      (output ? outputOptions : inputOptions).push(commonOptions);
    }
  );
  return [inputOptions, outputOptions];
};
const getLoaders = (options) => {
  const loaders = options.map(({ loader = "js" }) => loader === "default" ? "js" : loader);
  return [...new Set(loaders)];
};
const getExtension = (loader) => {
  switch (loader) {
    case "js":
    case "ts":
      return [loader, `c${loader}`, `m${loader}`];
    case "jsx":
    case "tsx":
    case "css":
    case "json":
      return loader;
  }
};
const getExtensions = (loaders) => {
  const extensions = [];
  loaders.forEach((loader) => {
    const extension = getExtension(loader);
    if (extension !== void 0) {
      extensions.push(...Array.isArray(extension) ? extension : [extension]);
    }
  });
  return extensions;
};
const getTransformOptions = (options) => options.map(({ include, exclude, ...transformOptions }) => transformOptions);
const getExtensionRegExp = (extension) => {
  if (extension === void 0) {
    return /^$/;
  } else {
    if (Array.isArray(extension)) {
      return new RegExp(`\\.(?:${extension.join("|")})$`);
    } else {
      return new RegExp(`\\.${extension}$`);
    }
  }
};
const getInputFilters = (options) => options.map(({ include, exclude = DEFAULT_EXCLUDE_REGEXP, loader = "js" }) => {
  const extension = getExtension(loader === "default" ? "js" : loader);
  return createFilter(include != null ? include : getExtensionRegExp(extension), exclude);
});
const getOutputFilters = (options) => options.map(({ include, exclude }) => createFilter(include, exclude));
const JS_EXTENSIONS = getExtensions(["js"]);
const JS_EXTENSION_REGEXP = getExtensionRegExp(JS_EXTENSIONS);
const resolveFilename = async (resolvedPath, extensions) => {
  const parsedPath = parse(resolvedPath);
  if (parsedPath.ext === "") {
    for (const extension of extensions) {
      const possibleFilename = `${resolvedPath}.${extension}`;
      try {
        await promises.access(possibleFilename);
        return possibleFilename;
      } catch {
      }
    }
  } else {
    const isJsExtension = JS_EXTENSION_REGEXP.test(parsedPath.ext);
    if (isJsExtension && extensions.includes("ts")) {
      const basename = join(parsedPath.dir, parsedPath.name);
      for (const extension of extensions) {
        const possibleFilename = `${basename}.${extension}`;
        try {
          await promises.access(possibleFilename);
          return possibleFilename;
        } catch {
        }
      }
    }
  }
  return null;
};
const getEsbuildTransformOptions = async (transformOptions, filters, id) => {
  let esbuildTransformOptions = null;
  for (let index = 0; index < transformOptions.length; index++) {
    if (!filters[index](id)) {
      continue;
    }
    const { loader, tsconfig, ...restTransformOptions } = transformOptions[index];
    const { tsconfigRaw } = restTransformOptions;
    if ((loader === "ts" || loader === "tsx") && tsconfigRaw === void 0 && tsconfig !== void 0) {
      const tsconfigPath = resolve(process.cwd(), tsconfig);
      restTransformOptions.tsconfigRaw = await promises.readFile(tsconfigPath, "utf8");
    }
    if (esbuildTransformOptions === null) {
      esbuildTransformOptions = { loader, ...restTransformOptions };
    } else {
      Object.assign(esbuildTransformOptions, restTransformOptions);
    }
  }
  return esbuildTransformOptions;
};
const handleTransformResult = async (pluginContext, { code, map, warnings }) => {
  if (warnings.length > 0) {
    const messages = await formatMessages(warnings, {
      kind: "warning",
      color: true
    });
    messages.forEach((message) => {
      pluginContext.warn(message);
    });
  }
  return {
    code,
    map: map === "" ? void 0 : map
  };
};
function esbuildTransform(options = {}) {
  const [inputOptions, outputOptions] = splitOptionsByType(options);
  const loaders = getLoaders(inputOptions);
  const extensions = getExtensions(loaders);
  const scriptLoaders = loaders.filter((loader) => SCRIPT_LOADERS.includes(loader));
  const scriptExtensions = getExtensions(scriptLoaders);
  const inputTransformOptions = getTransformOptions(inputOptions);
  const outputTransformOptions = getTransformOptions(outputOptions);
  const inputFilters = getInputFilters(inputOptions);
  const outputFilters = getOutputFilters(outputOptions);
  return {
    name: "esbuild-transform",
    async resolveId(source, importer) {
      if (importer === void 0 || !(source.startsWith(".") || isAbsolute(source))) {
        return null;
      }
      const resolvedPath = resolve(dirname(importer), source);
      try {
        return (await promises.stat(resolvedPath)).isDirectory() ? await resolveFilename(join(resolvedPath, "index"), scriptExtensions) : resolvedPath;
      } catch {
        return await resolveFilename(resolvedPath, extensions);
      }
    },
    async transform(code, id) {
      const esbuildTransformOptions = await getEsbuildTransformOptions(
        inputTransformOptions,
        inputFilters,
        id
      );
      if (esbuildTransformOptions === null) {
        return null;
      }
      const transformResult = await transform(code, {
        format: esbuildTransformOptions.loader === "json" ? "esm" : void 0,
        sourcefile: id,
        sourcemap: true,
        ...esbuildTransformOptions
      });
      return await handleTransformResult(this, transformResult);
    },
    async renderChunk(code, chunk) {
      const esbuildTransformOptions = await getEsbuildTransformOptions(
        outputTransformOptions,
        outputFilters,
        chunk.fileName
      );
      if (esbuildTransformOptions === null) {
        return null;
      }
      const transformResult = await transform(code, {
        sourcefile: chunk.fileName,
        sourcemap: true,
        ...esbuildTransformOptions
      });
      return await handleTransformResult(this, transformResult);
    }
  };
}
export { esbuildTransform as default };
